import typer
import requests
import toml
import os
import causadb.cli.utils as utils

CAUSADB_URL = utils.CAUSADB_URL

app = typer.Typer()


@app.command()
def setup():
    """
    Set up a CausaDB account on this device
    """
    token_id = typer.prompt(
        "Token ID")
    token_secret = typer.prompt(
        "Token secret")

    headers = {"token": token_secret}

    response = requests.get(
        f"{CAUSADB_URL}/account",
        headers=headers
    )

    if response.status_code == 200:
        typer.echo(
            "Setup successful, saving credentials to ~/.causadb/config.toml.")

        # Save the token to ~/.causadb/config.toml. If the file doesn't exist, create it.
        dir_name = os.path.expanduser("~/.causadb")
        os.makedirs(os.path.expanduser("~/.causadb"), exist_ok=True)
        config_filepath = os.path.join(dir_name, "config.toml")

        config = {}
        if os.path.exists(config_filepath):
            with open(config_filepath, "r") as f:
                config = toml.load(f)

        config["default"] = {
            "token_id": token_id,
            "token_secret": token_secret,
        }

        with open(config_filepath, "w") as f:
            toml.dump(config, f)

    else:
        typer.echo("Setup failed, please check your credentials and try again.")


@app.command()
def remove():
    """
    Remove CausaDB account from this device
    """
    # Get config from ~/.causadb/config.toml
    dir_name = os.path.expanduser("~/.causadb")
    config_filepath = os.path.join(dir_name, "config.toml")

    if not os.path.exists(config_filepath):
        typer.echo(
            "No profiles found in ~/.causadb/config.toml.")
        return 1

    # Confirm removal
    confirm = typer.confirm(
        f"Are you sure you want to remove your account from this device? (The account will not be deleted from the cloud.)")

    if confirm:
        # Delete the token from ~/.causadb/config.toml.

        with open(config_filepath, "w") as f:
            toml.dump({}, f)

        typer.echo(
            f"Account successfully removed from this device. You can add it again with `causadb account setup`.")


@app.command()
def info():
    """
    Show information about the account
    """
    # Get config from ~/.causadb/config.toml
    dir_name = os.path.expanduser("~/.causadb")
    config_filepath = os.path.join(dir_name, "config.toml")

    if not os.path.exists(config_filepath):
        typer.echo(
            "No config found in ~/.causadb/config.toml.")
        return 1

    # Get the token from ~/.causadb/config.toml.
    with open(config_filepath, "r") as f:
        config = toml.load(f)

    org_id = config[list(config.keys())[0]]["org_id"]
    token = config[list(config.keys())[0]]["token"]

    typer.echo(f"Organization ID: {org_id}")
    typer.echo(f"Token: {token}")



import requests
import time
import pandas as pd
import numpy as np
from typing import Union

from .utils import get_causadb_url


class Model:
    def __init__(self, model_name: str, client: "CausaDB") -> None:
        """Initializes the Model class.

        Args:
            model_name (str): The name of the model.
            client (CausaDB): A CausaDB client.
        """
        self.client = client
        self.model_name = model_name
        self.config = {}

        # Pull config from the server
        headers = {"token": self.client.token_secret}
        response = requests.get(
            f"{get_causadb_url()}/models/{self.model_name}",
            headers=headers,
        ).json()

        if "details" in response:
            self.config = response["details"]["config"]

        self._update()

    def __repr__(self) -> str:
        return f"<Model {self.model_name}>"

    def remove(self) -> None:
        """Remove the model from the CausaDB system."""
        headers = {"token": self.client.token_secret}
        try:
            requests.delete(
                f"{get_causadb_url()}/models/{self.model_name}",
                headers=headers,
            )
        except:
            raise Exception("CausaDB server request failed")

    def set_nodes(self, nodes: list[str]) -> None:
        """Set the nodes of the model.

        Args:
            nodes (list[str]): A list of node names.

        Example:
            >>> model.set_nodes(["x", "y", "z"])
        """
        headers = {"token": self.client.token_secret}

        try:
            response = requests.get(
                f"{get_causadb_url()}/models/{self.model_name}", headers=headers
            ).json()
        except:
            raise Exception("CausaDB server request failed")

        self.config = response["details"]["config"]
        self.config["nodes"] = nodes

        self._update()

    def get_nodes(self) -> list[str]:
        """Get the nodes of the model.

        Returns:
            list[str]: A list of node names.
        """
        headers = {"token": self.client.token_secret}

        try:
            response = requests.get(
                f"{get_causadb_url()}/models/{self.model_name}", headers=headers
            ).json()
        except:
            raise Exception("CausaDB server request failed")

        return response["details"]["config"]["nodes"]

    def set_edges(self, edges: list[tuple[str, str]]) -> None:
        """Set the edges of the model.

        Args:
            edges (list[tuple[str, str]]): A list of tuples representing edges.

        Example:
            >>> model.set_edges([
            ...     ("SaturatedFatsInDiet", "Weight"),
            ...     ("Weight", "BMI"),
            ... ])
        """
        headers = {"token": self.client.token_secret}

        try:
            response = requests.get(
                f"{get_causadb_url()}/models/{self.model_name}", headers=headers
            ).json()
        except:
            raise Exception("CausaDB server request failed")

        self.config = response["details"]["config"]
        self.config["edges"] = edges

        self._update()

    def get_edges(self) -> list[tuple[str, str]]:
        """Get the edges of the model.

        Returns:
            list[tuple[str, str]]: A list of tuples representing edges.
        """
        headers = {"token": self.client.token_secret}

        try:
            response = requests.get(
                f"{get_causadb_url()}/models/{self.model_name}", headers=headers
            ).json()
        except:
            raise Exception("CausaDB server request failed")

        edges = response["details"]["config"]["edges"]
        # Convert the edges to a list of tuples
        return [(edge[0], edge[1]) for edge in edges]

    def set_node_types(self, node_types: dict) -> None:
        """Set the node types of the model.

        Args:
            node_types (dict): A dictionary of node types.

        Example:
            >>> model.set_node_types({
            ...     "x1": {"type": "seasonal", "min": 0, "max": 1}
            ... })
        """
        headers = {"token": self.client.token_secret}

        try:
            response = requests.get(
                f"{get_causadb_url()}/models/{self.model_name}", headers=headers
            ).json()
        except:
            raise Exception("CausaDB server request failed")

        self.config = response["details"]["config"]
        self.config["node_types"] = node_types

        self._update()

    def get_node_types(self) -> dict:
        """Get the node types of the model.

        Returns:
            dict: A dictionary of node types.
        """
        headers = {"token": self.client.token_secret}

        try:
            response = requests.get(
                f"{get_causadb_url()}/models/{self.model_name}", headers=headers
            ).json()
        except:
            raise Exception("CausaDB server request failed")

        return response["details"]["config"]["node_types"]

    def attach(self, data_name: str) -> None:
        """Attach data to the model.

        Args:
            data_name (str): The name of the data to attach.
        """
        headers = {"token": self.client.token_secret}

        try:
            response = requests.post(
                f"{get_causadb_url()}/models/{self.model_name}/attach/{data_name}",
                headers=headers
            ).json()
        except:
            raise Exception("CausaDB server request failed")

    def detach(self, data_name: str) -> None:
        """Detach data from the model.

        Args:
            data_name (str): The name of the data to detach.
        """
        headers = {"token": self.client.token_secret}

        try:
            response = requests.delete(
                f"{get_causadb_url()}/models/{self.model_name}/detach",
                headers=headers
            ).json()
        except:
            raise Exception("CausaDB server request failed")

    def train(self, wait: bool = True, poll_interval: float = 0.2, poll_limit: float = 30) -> None:
        """Train the model.

        Args:
            wait (bool): Whether to wait for the model to finish training.
            poll_interval (float): The interval at which to poll the server for the model status.
            poll_limit (float): The maximum time to wait for the model to finish training.

        Example:
            >>> model.train()
        """
        headers = {"token": self.client.token_secret}

        try:
            response = requests.post(
                f"{get_causadb_url()}/models/{self.model_name}/train",
                headers=headers
            )
        except:
            raise Exception("CausaDB server request failed")

        # If HTTPException status code is 400, raise an exception
        if response.status_code == 400:
            raise Exception(response.json()["detail"])
        response = response.json()

        if wait:
            time_elapsed = 0
            while self.status() != "trained":
                time.sleep(poll_interval)
                # If the model takes too long to train, raise an exception
                time_elapsed += poll_interval
                if time_elapsed > poll_limit:
                    raise Exception("Model training took too long")

    def status(self) -> str:
        """Get the status of the model.

        Returns:
            str: The status of the model.
        """
        headers = {"token": self.client.token_secret}

        try:
            response = requests.get(
                f"{get_causadb_url()}/models/{self.model_name}",
                headers=headers,
            ).json()
        except:
            raise Exception("CausaDB server request failed")

        model_status = response["details"]["status"]

        return model_status

    def simulate_actions(self, actions: dict, fixed: dict = {}, interval: float = 0.9, observation_noise: bool = False) -> dict:
        """Simulate an action on the model.

        Args:
            actions (dict): A dictionary representing the actions.
            fixed (dict): A dictionary representing the fixed nodes.
            interval (float): The interval at which to simulate the action.
            observation_noise (bool): Whether to include observation noise.

        Returns:
            dict: A dictionary representing the result of the action.

        Example:
            >>> model.simulate_actions(
            ...     {"x": [0, 1]}
            ... )
        """
        headers = {"token": self.client.token_secret}

        query = {
            "actions": actions,
            "fixed": fixed,
            "interval": interval,
            "observation_noise": observation_noise
        }

        try:
            response = requests.post(
                f"{get_causadb_url()}/models/{self.model_name}/simulate-actions",
                headers=headers,
                json=query,
            ).json()
        except:
            raise Exception("CausaDB server request failed")

        if "outcome" in response:
            outcome = response["outcome"]
            return {
                "median": pd.DataFrame.from_dict(outcome["median"]),
                "lower": pd.DataFrame.from_dict(outcome["lower"]),
                "upper": pd.DataFrame.from_dict(outcome["upper"])
            }

        raise Exception("CausaDB server request failed")

    def causal_effects(self, actions: Union[str, dict[str, tuple[np.ndarray, np.ndarray]]], fixed: dict[str, np.ndarray] = None, interval: float = 0.90, observation_noise=False) -> pd.DataFrame:
        """ Get the causal effects of actions on the model.

        Args:
            actions (Union[str, dict[str, tuple[np.ndarray, np.ndarray]]]): A dictionary representing the actions.
            fixed (dict): A dictionary representing the fixed nodes.
            interval (float): The interval at which to simulate the action.
            observation_noise (bool): Whether to include observation noise.

        Returns:
            pd.DataFrame: A dataframe representing the causal effects of the actions.

        Example:
            >>> model.causal_effects(
            ...     {"x": [0, 1]}
            ... )

        """
        headers = {"token": self.client.token_secret}

        query = {
            "actions": actions,
            "fixed": fixed,
            "interval": interval,
            "observation_noise": observation_noise
        }

        try:
            response = requests.post(
                f"{get_causadb_url()}/models/{self.model_name}/causal-effects",
                headers=headers,
                json=query,
            ).json()
        except:
            raise Exception("CausaDB server request failed")

        if "outcome" in response:
            return pd.DataFrame.from_dict(response["outcome"])

        raise Exception("CausaDB server request failed")

    def find_best_actions(self, targets: dict[str, float], actionable: list[str], fixed: dict[str, float] = {}) -> dict:
        """Get the optimal actions for a given set of target outcomes.

        Args:
            targets (dict[str, float]): A dictionary representing the target outcomes.
            actionable (list[str]): A list of actionable nodes.
            fixed (dict[str, float]): A dictionary representing the fixed nodes.

        Returns:
            dict: A dictionary representing the optimal actions.

        Example:
            >>> model.optimal_actions(
            ...     {"x": 0.5},
            ...     ["x"],
            ...     {"y": 0.5}
        """
        headers = {"token": self.client.token_secret}

        query = {
            "targets": targets,
            "actionable": actionable,
            "fixed": fixed
        }

        try:
            response = requests.post(
                f"{get_causadb_url()}/models/{self.model_name}/find-best-actions",
                headers=headers,
                json=query,
            ).json()
        except:
            raise Exception("CausaDB server request failed")

        if "best_actions" in response:
            return pd.DataFrame.from_dict(response["best_actions"])

        raise Exception("CausaDB server request failed")

    def causal_attributions(self, outcome: str, normalise: bool = False) -> pd.DataFrame:
        """Get the causal attributions for an outcome.

        Args:
            outcome (str): The outcome node.
            normalise (bool): Whether to normalise the causal attributions.

        Returns:
            pd.DataFrame: A dataframe representing the causal attributions of the outcome.

        Example:
            >>> model.causal_attributions("y")
        """
        headers = {"token": self.client.token_secret}

        query = {
            "outcome": outcome,
            "normalise": normalise
        }

        try:
            response = requests.post(
                f"{get_causadb_url()}/models/{self.model_name}/causal-attributions",
                headers=headers,
                json=query,
            ).json()
        except:
            raise Exception("CausaDB server request failed")

        if "outcome" in response:
            return pd.DataFrame.from_dict(response["outcome"])

        raise Exception("CausaDB server request failed")

    def _update(self) -> None:
        """Pushes the current state of the model to the CausaDB server."""
        headers = {"token": self.client.token_secret}

        try:
            requests.post(
                f"{get_causadb_url()}/models/{self.model_name}",
                headers=headers,
                json=self.config
            ).json()
        except:
            raise Exception("CausaDB server request failed")



import typer
from typing import Optional
import causadb.cli.account as account
import causadb.cli.data as data
import causadb.cli.models as models
from causadb import __version__

app = typer.Typer()


def version_callback(value: bool):
    if value:
        typer.echo(f"CausaDB CLI v{__version__}")
        raise typer.Exit()


@app.callback()
def cli(
        _: Optional[bool] = typer.Option(
            None, "-v", "--version", callback=version_callback, is_eager=True, help="Print the version of the CausaDB CLI"
        )
):
    ...


app.add_typer(account.app, name="account", help="Manage account")
app.add_typer(data.app, name="data", help="Manage linked datasources")
app.add_typer(models.app, name="models", help="Manage models")



import requests
import pandas as pd
from .utils import get_causadb_url


class Data:
    def __repr__(self) -> str:
        return f"<Data {self.data_name}>"

    def __init__(self, data_name: str, client: "CausaDB") -> None:
        """Initializes the Data class.

        Args:
            data_name (str): The name of the data.
            client (CausaDB): A CausaDB client.
        """
        self.data_name = data_name
        self.client = client

    def remove(self) -> None:
        """Remove the data from the CausaDB system."""
        headers = {"token": self.client.token_secret}
        try:
            requests.delete(
                f"{get_causadb_url()}/data/{self.data_name}",
                headers=headers,
            )
        except:
            raise Exception("CausaDB server request failed")

    def from_csv(self, filepath: str) -> None:
        """Add data from a CSV file.

        Args:
            filepath (str): The path to the CSV file.
        """
        dataset = pd.read_csv(filepath).to_dict()

        self._update(dataset)

    def _update(self, data: dict) -> None:
        """Pushes the data to the CausaDB server.

        Args:
            data (dict): The new data.
        """

        # Send a POST request to the CausaDB server to update the data
        try:
            headers = {"token": self.client.token_secret}
            response = requests.post(
                f"{get_causadb_url()}/data/{self.data_name}",
                headers=headers,
                json=data,
            ).json()
        except:
            raise Exception("CausaDB client failed to connect to server")

        if response["status"] != "success":
            # If the response is not successful, raise an exception and include the error message
            raise Exception(f"Failed to update data: {response['message']}")



import typer
import requests
from causadb.cli.utils import load_config, show_table, CAUSADB_URL
import pandas as pd
from typing import Annotated

app = typer.Typer()


@app.command()
def list():
    """
    List linked datasources.
    """
    config = load_config()
    token_secret = config["default"]["token_secret"]

    headers = {"token": token_secret}

    data = requests.get(
        f"{CAUSADB_URL}/data", headers=headers
    ).json()

    show_table(data["data"], columns=["id", "name", "type"])


@app.command()
def add(
    filepath: Annotated[str, typer.Option(
        help="The path to your data file.")] = None,
    name: Annotated[str, typer.Option(
        "--data",
        help="The name to give your new data source.")] = None
):
    """
    Add a datasource.
    """

    # If filepath is None, prompt the user for a filepath.
    if filepath is None:
        typer.echo(
            "Note: We only support CSV files at the moment, but more formats will be supported soon.")
        filepath = typer.prompt(
            "Enter the path to your datasource file (e.g. /path/to/file.csv)")

    # Read the file into a pandas dataframe.
    try:
        dataset = pd.read_csv(filepath).to_dict()
    except:
        typer.echo("Failed to read file.")
        return 1

    # If dataset name is None, prompt the user for a name.
    data_name = name
    if data_name is None:
        default_name = filepath.split("/")[-1].split(".")[0]

        data_name = typer.prompt(
            "Enter a name for your datasource", default=default_name)

    config = load_config()
    token_secret = config["default"]["token_secret"]

    headers = {"token": token_secret}

    data = requests.post(
        f"{CAUSADB_URL}/data/{data_name}",
        headers=headers,
        json=dataset,
    ).json()

    if data["status"] == "success":
        typer.echo("Successfully added datasource.")

    else:
        typer.echo("Failed to add datasource.")


@app.command()
def remove(
    name: Annotated[str, typer.Option(
        "--data",
        help="Name of the data source you want to remove.")] = None
):
    """
    Remove a datasource.
    """

    if name is None:
        # Prompt the user for the name of the datasource to remove
        name = typer.prompt(
            "Enter the name of the datasource you want to remove")

    # Load the configuration and get the token secret
    config = load_config()
    token_secret = config["default"]["token_secret"]

    headers = {"token": token_secret}

    # Make a request to the remove data endpoint
    response = requests.delete(
        f"{CAUSADB_URL}/data/{name}",
        headers=headers
    )

    # Check the response status and print appropriate message
    if response.status_code == 200:
        data = response.json()
        if data["status"] == "success":
            typer.echo("Successfully removed datasource.")
        else:
            typer.echo(f"Failed to remove datasource: {data['message']}")
    else:
        typer.echo(
            f"Failed to remove datasource. Server responded with status code {response.status_code}.")



import os
from dotenv import load_dotenv
load_dotenv()


def get_causadb_url():
    return os.getenv("CAUSADB_URL", "https://api.causadb.com/v1")


def set_causadb_url(url: str):
    os.environ["CAUSADB_URL"] = url
